package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.TFFlowApprove;
import com.qmth.distributed.print.business.entity.TFFlowLog;
import com.qmth.distributed.print.business.service.ActivitiService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.service.TFFlowApproveService;
import com.qmth.distributed.print.business.service.TFFlowLogService;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowNode;
import org.activiti.bpmn.model.UserTask;
import org.activiti.engine.HistoryService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.impl.RepositoryServiceImpl;
import org.activiti.engine.impl.persistence.entity.ProcessDefinitionEntity;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: activiti service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/12
 */
@Service
public class ActivitiServiceImpl implements ActivitiService {
    private final static Logger log = LoggerFactory.getLogger(ActivitiServiceImpl.class);

    @Resource
    private RuntimeService runtimeService;

    @Resource
    private TaskService taskService;

    @Resource
    private RepositoryService repositoryService;

    @Resource
    TFFlowApproveService tfFlowApproveService;

    @Resource
    TFFlowLogService tfFlowLogService;

    @Resource
    HistoryService historyService;

    @Resource
    ExamTaskService examTaskService;

    /**
     * 注册流程
     *
     * @param processFileName
     */
    @Override
    public void createDeployment(String processFileName) {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addClasspathResource(SystemConstant.PROCESSES + File.separator + processFileName);
        builder.deploy();
    }

    /**
     * 上传流程文件
     *
     * @param file
     */
    @Override
    public void uploadDeployment(MultipartFile file) throws IOException {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addInputStream(file.getOriginalFilename(), file.getInputStream());
        builder.deploy();
    }

    /**
     * 查询待办
     */
    @Override
    public void getTaskList() {
        //获取待办的一些信息，这里可以传入需要查询的用户，
        //我这里查询的所有待办
//        List<Task> tasks = taskService.createTaskQuery().taskAssignee("test123").orderByTaskCreateTime().desc().list();
        List<Task> tasks = taskService.createTaskQuery().list();
//        Task task1 = taskService.createTaskQuery().processInstanceId("22531").singleResult();
//        Task task1 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("赵总")//设置候选用户
//                .singleResult();
//        Task task2 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("王总")//设置候选用户
//                .singleResult();
//        Task task3 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("王总1")//设置候选用户
//                .singleResult();
//        Task task4 = taskService.createTaskQuery()
//                .processDefinitionKey("myProcess3")
//                .taskCandidateUser("赵总1")//设置候选用户
//                .singleResult();
        for (Task t : tasks) {
            log.info("assignee:{},createTime:{},id:{},getProcessDefinitionId:{},name:{},processInstanceId:{},taskDefinitionKey:{},parentTaskId:{}", t.getAssignee(), t.getCreateTime(), t.getId(), t.getProcessDefinitionId(), t.getName(), t.getProcessInstanceId(), t.getTaskDefinitionKey(), t.getParentTaskId());
        }
    }

    /**
     * 根据流程key开启一个流程
     *
     * @param processKey
     * @param map
     */
    @Override
    public ProcessInstance startActivity(String processKey, Map<String, Object> map) {
        ProcessInstance processInstance = null;
        if (Objects.nonNull(map) && map.size() > 0) {
            processInstance = runtimeService.startProcessInstanceByKey(processKey, map);
        } else {
            processInstance = runtimeService.startProcessInstanceByKey(processKey);
        }
        log.info("流程id:{},processInstanceId:{},processDefinitionId:{},processDefinitionKey:{}", processInstance.getId(), processInstance.getProcessInstanceId(), processInstance.getProcessDefinitionId(), processInstance.getProcessDefinitionKey());
        ProcessInstance finalProcessInstance = processInstance;
        map.computeIfAbsent(SystemConstant.FLOW_ID, v -> finalProcessInstance.getId());
        map.computeIfAbsent(SystemConstant.FLOW_STATUS, v -> FlowStatusEnum.START);
        ActivitiService activitiService = SpringContextHolder.getBean(ActivitiService.class);
        activitiService.flowStart(map);
        return processInstance;
    }

    /**
     * 根据任务id提交任务
     *
     * @param taskId
     */
    @Override
    public void complete(String taskId) {
        Map<String, Object> map = new HashMap<>();
        map.put("user", 1);
        map.put("approve", 1);
        map.put("assignee", 0);
        taskService.complete(taskId, map);
    }

    /**
     * 根据流程id直接结束流程
     *
     * @param flowId
     */
    @Override
    public void deleteProcessInstance(String flowId) {
        runtimeService.deleteProcessInstance(flowId, "结束");
    }

    /**
     * 流程启动
     *
     * @param map
     */
    @Override
    @Transactional
    public void flowStart(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        if (Objects.isNull(map.get(SystemConstant.FLOW_ID))) {
            throw ExceptionResultEnum.ERROR.exception("流程id不能为空");
        }
        Long flowId = Long.parseLong(map.get(SystemConstant.FLOW_ID).toString());

        if (Objects.isNull(map.get(SystemConstant.FLOW_STATUS))) {
            throw ExceptionResultEnum.ERROR.exception("流程状态不能为空");
        }
        FlowStatusEnum flowStatusEnum = (FlowStatusEnum) map.get(SystemConstant.FLOW_STATUS);

        Long approveId = Objects.nonNull(map.get(SystemConstant.APPROVE_ID)) ? Long.parseLong(String.valueOf(map.get(SystemConstant.APPROVE_ID))) : null;
        tfFlowApproveService.save(new TFFlowApprove(flowId, approveId, flowStatusEnum, sysUser.getId()));
    }

    /**
     * 审批任务
     *
     * @param map
     */
    @Override
    @Transactional
    public void taskApprove(Map<String, Object> map) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(map.get(SystemConstant.FLOW_TASK_ID))) {
            throw ExceptionResultEnum.ERROR.exception("流程任务id不能为空");
        }
        String flowTaskId = (String) map.get(SystemConstant.FLOW_TASK_ID);
        Task task = taskService.createTaskQuery().taskId(flowTaskId).singleResult();
        if (Objects.isNull(task)) {
            throw ExceptionResultEnum.ERROR.exception("流程任务为空");
        }
        String processInstanceId = task.getProcessInstanceId();
        QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
        examTaskQueryWrapper.lambda().eq(ExamTask::getFlowId, Long.parseLong(processInstanceId));
        ExamTask examTask = examTaskService.getOne(examTaskQueryWrapper);

        String processDefinitionId = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId).singleResult().getProcessDefinitionId();
        ProcessDefinitionEntity processDefinitionEntity = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService).getDeployedProcessDefinition(processDefinitionId);
        String currActivityId = task.getTaskDefinitionKey();
        BpmnModel bpmnModel = repositoryService.getBpmnModel(task.getProcessDefinitionId());
        FlowNode currFlow = (FlowNode) bpmnModel.getMainProcess().getFlowElement(currActivityId);
        QueryWrapper<TFFlowApprove> tfFlowApproveQueryWrapper = new QueryWrapper<>();
        tfFlowApproveQueryWrapper.lambda().eq(TFFlowApprove::getFlowId, processInstanceId);
        TFFlowApprove tfFlowApprove = tfFlowApproveService.getOne(tfFlowApproveQueryWrapper);
        TFFlowLog tfFlowLog = null;
        if (currFlow instanceof UserTask) {
            UserTask userTask = (UserTask) currFlow;
            //广东医科大学流程
            if (Objects.nonNull(processDefinitionEntity) && processDefinitionEntity.getKey().contains(SystemConstant.GDYKDX_FLOW_KEY)) {
                String remark = Objects.nonNull(map.get(SystemConstant.APPROVE_REMARK)) ? map.get(SystemConstant.APPROVE_REMARK).toString() : null;
                tfFlowLog = new TFFlowLog(Long.parseLong(processInstanceId), examTask.getId(), sysUser.getId(), sysUser.getId(), remark);
                FlowGdykdxApproveSetupEnum setupEnum = FlowGdykdxApproveSetupEnum.convertToInstance(userTask.getId());
                if (setupEnum == FlowGdykdxApproveSetupEnum.SUBMIT) {//命题提交
                    Task taskUser = taskService.createTaskQuery().taskId(flowTaskId).taskAssignee(String.valueOf(sysUser.getId())).singleResult();
                    if (Objects.isNull(taskUser)) {
                        throw ExceptionResultEnum.ERROR.exception("不能提交他人的任务");
                    }
                    tfFlowLog.setApproveSetup(FlowGdykdxApproveSetupEnum.SUBMIT.getSetup());
                    tfFlowLog.setApproveOperation(FlowApproveOperationEnum.SUBMIT);
                    tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                    tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE.getSetup());
                } else if (setupEnum == FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE) {//主任提交
                    Task taskUser = taskService.createTaskQuery().taskId(flowTaskId).taskCandidateUser(String.valueOf(sysUser.getId())).singleResult();
                    if (Objects.isNull(taskUser)) {
                        throw ExceptionResultEnum.ERROR.exception("不能提交他人的任务");
                    }
                    if (Objects.isNull(map.get(SystemConstant.APPROVE_OPERATION))) {
                        throw ExceptionResultEnum.ERROR.exception("流程审核操作不能为空");
                    }
                    FlowApprovePassEnum approvePass = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_OPERATION);
                    String approve = approvePass == FlowApprovePassEnum.PASS ? "1" : "0";
                    tfFlowLog.setApproveSetup(FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE.getSetup());
                    if (Objects.equals(approve, "0")) {//驳回命题
                        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
                        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.SUBMIT.getSetup());
                    } else if (Objects.equals(approve, "1")) {//提交
                        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
                        tfFlowApprove.setStatus(FlowStatusEnum.AUDITING);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.SECOND_APPROVE.getSetup());
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
                    }
                    map.computeIfAbsent(SystemConstant.APPROVE, v -> approve);
                } else if (setupEnum == FlowGdykdxApproveSetupEnum.SECOND_APPROVE) {//院长提交
                    Task taskUser = taskService.createTaskQuery().taskId(flowTaskId).taskCandidateUser(String.valueOf(sysUser.getId())).singleResult();
                    if (Objects.isNull(taskUser)) {
                        throw ExceptionResultEnum.ERROR.exception("不能提交他人的任务");
                    }
                    if (Objects.isNull(map.get(SystemConstant.APPROVE_OPERATION))) {
                        throw ExceptionResultEnum.ERROR.exception("流程审核操作不能为空");
                    }
                    FlowApprovePassEnum approvePass = (FlowApprovePassEnum) map.get(SystemConstant.APPROVE_OPERATION);
                    String approve = null;
                    if (approvePass == FlowApprovePassEnum.PASS) {
                        approve = "2";
                    } else {
                        if (Objects.isNull(map.get(SystemConstant.APPROVE_SETUP))) {
                            throw ExceptionResultEnum.ERROR.exception("流程驳回节点不能为空");
                        }
                        Integer setup = Integer.parseInt(map.get(SystemConstant.APPROVE_SETUP).toString());
                        if (setup.intValue() < FlowGdykdxApproveSetupEnum.SUBMIT.getSetup() || setup.intValue() >= FlowGdykdxApproveSetupEnum.SECOND_APPROVE.getSetup()) {
                            throw ExceptionResultEnum.ERROR.exception("流程驳回节点有误");
                        }
                        approve = setup.intValue() == 1 ? "0" : "1";
                    }
                    tfFlowLog.setApproveSetup(FlowGdykdxApproveSetupEnum.SECOND_APPROVE.getSetup());
                    if (Objects.equals(approve, "0")) {//驳回命题
                        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
                        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.SUBMIT.getSetup());
                    } else if (Objects.equals(approve, "1")) {//驳回主任
                        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.REJECT);
                        tfFlowApprove.setStatus(FlowStatusEnum.REJECT);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.PRIMARY_APPROVE.getSetup());
                    } else if (Objects.equals(approve, "2")) {//提交
                        tfFlowLog.setApproveOperation(FlowApproveOperationEnum.APPROVE);
                        tfFlowApprove.setStatus(FlowStatusEnum.FINISH);
                        tfFlowApprove.setSetup(FlowGdykdxApproveSetupEnum.END.getSetup());
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("流程变量值错误");
                    }
                    String finalApprove = approve;
                    map.computeIfAbsent(SystemConstant.APPROVE, v -> finalApprove);
                }
                tfFlowApprove.updateInfo(sysUser.getId());
            }
        }
        tfFlowApprove.setApproveId(sysUser.getId());
        //当前实例的执行到哪个节点
        taskService.complete(String.valueOf(flowTaskId), map);
        tfFlowApproveService.saveOrUpdate(tfFlowApprove);
        if (Objects.nonNull(tfFlowLog)) {
            tfFlowLogService.save(tfFlowLog);
        }
    }
}
